@@ -37,98 +37,21 @@
 extern zend_class_entry *redis_ce;
 zend_class_entry *redis_array_ce;
 
-ZEND_BEGIN_ARG_INFO_EX(arginfo_ctor, 0, 0, 1)
-    ZEND_ARG_INFO(0, name_or_hosts)
-    ZEND_ARG_ARRAY_INFO(0, options, 0)
-ZEND_END_ARG_INFO()
+#if PHP_VERSION_ID < 80000
+#include "redis_array_legacy_arginfo.h"
+#else
+#include "zend_attributes.h"
+#include "redis_array_arginfo.h"
+#endif
 
-ZEND_BEGIN_ARG_INFO_EX(arginfo_call, 0, 0, 2)
-    ZEND_ARG_INFO(0, function_name)
-    ZEND_ARG_INFO(0, arguments)
-ZEND_END_ARG_INFO()
+PHP_MINIT_FUNCTION(redis_array)
+{
+    /* RedisSentinel class */
+    redis_array_ce = register_class_RedisArray();
+    redis_array_ce->create_object = create_redis_array_object;
 
-ZEND_BEGIN_ARG_INFO_EX(arginfo_target, 0, 0, 1)
-    ZEND_ARG_INFO(0, key)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_instance, 0, 0, 1)
-    ZEND_ARG_INFO(0, host)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_rehash, 0, 0, 0)
-    ZEND_ARG_INFO(0, callable)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_select, 0, 0, 1)
-    ZEND_ARG_INFO(0, index)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_mget, 0, 0, 1)
-    ZEND_ARG_INFO(0, keys)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_mset, 0, 0, 1)
-    ZEND_ARG_INFO(0, pairs)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_del, 0, 0, 1)
-    ZEND_ARG_INFO(0, keys)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_getopt, 0, 0, 1)
-    ZEND_ARG_INFO(0, opt)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_setopt, 0, 0, 2)
-    ZEND_ARG_INFO(0, opt)
-    ZEND_ARG_INFO(0, value)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_keys, 0, 0, 1)
-    ZEND_ARG_INFO(0, pattern)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_multi, 0, 0, 1)
-    ZEND_ARG_INFO(0, host)
-    ZEND_ARG_INFO(0, mode)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_flush, 0, 0, 0)
-    ZEND_ARG_INFO(0, async)
-ZEND_END_ARG_INFO()
-
-zend_function_entry redis_array_functions[] = {
-     PHP_ME(RedisArray, __call, arginfo_call, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, __construct, arginfo_ctor, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, _continuum, arginfo_void, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, _distributor, arginfo_void, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, _function, arginfo_void, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, _hosts, arginfo_void, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, _instance, arginfo_instance, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, _rehash, arginfo_rehash, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, _target, arginfo_target, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, bgsave, arginfo_void, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, del, arginfo_del, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, discard, arginfo_void, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, exec, arginfo_void, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, flushall, arginfo_flush, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, flushdb, arginfo_flush, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, getOption, arginfo_getopt, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, info, arginfo_void, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, keys, arginfo_keys, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, mget, arginfo_mget, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, mset, arginfo_mset, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, multi, arginfo_multi, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, ping, arginfo_void, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, save, arginfo_void, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, select, arginfo_select, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, setOption,arginfo_setopt, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, unlink, arginfo_void, ZEND_ACC_PUBLIC)
-     PHP_ME(RedisArray, unwatch, arginfo_void, ZEND_ACC_PUBLIC)
-     PHP_MALIAS(RedisArray, delete, del, arginfo_del, ZEND_ACC_PUBLIC)
-     PHP_MALIAS(RedisArray, getMultiple, mget, arginfo_mget, ZEND_ACC_PUBLIC)
-     PHP_FE_END
-};
+    return SUCCESS;
+}
 
 static void
 redis_array_free(RedisArray *ra)
@@ -1195,6 +1118,86 @@ PHP_METHOD(RedisArray, unlink) {
     ra_generic_del(INTERNAL_FUNCTION_PARAM_PASSTHRU, "UNLINK", sizeof("UNLINK") - 1);
 }
 
+static void
+ra_generic_scan_cmd(INTERNAL_FUNCTION_PARAMETERS, const char *kw, int kw_len)
+{
+    RedisArray *ra;
+    zend_string *key, *pattern = NULL;
+    zval *object, *redis_inst, *z_iter, z_fun, z_args[4];
+    zend_long count = 0;
+
+    if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "OSz/|S!l",
+            &object, redis_array_ce, &key, &z_iter, &pattern, &count) == FAILURE) {
+        RETURN_FALSE;
+    }
+
+    if ((ra = redis_array_get(object)) == NULL) {
+        RETURN_FALSE;
+    }
+
+    if ((redis_inst = ra_find_node(ra, ZSTR_VAL(key), ZSTR_LEN(key), NULL)) == NULL) {
+        php_error_docref(NULL, E_ERROR, "Could not find any redis servers for this key.");
+        RETURN_FALSE;
+    }
+
+    ZVAL_STR(&z_args[0], key);
+    ZVAL_NEW_REF(&z_args[1], z_iter);
+    if (pattern) ZVAL_STR(&z_args[2], pattern);
+    ZVAL_LONG(&z_args[3], count);
+
+    ZVAL_STRINGL(&z_fun, kw, kw_len);
+    call_user_function(&redis_ce->function_table, redis_inst, &z_fun, return_value, ZEND_NUM_ARGS(), z_args);
+    zval_dtor(&z_fun);
+
+    ZVAL_ZVAL(z_iter, &z_args[1], 0, 1);
+}
+
+PHP_METHOD(RedisArray, hscan)
+{
+    ra_generic_scan_cmd(INTERNAL_FUNCTION_PARAM_PASSTHRU, "HSCAN", sizeof("HSCAN") - 1);
+}
+
+PHP_METHOD(RedisArray, sscan)
+{
+    ra_generic_scan_cmd(INTERNAL_FUNCTION_PARAM_PASSTHRU, "SSCAN", sizeof("SSCAN") - 1);
+}
+
+PHP_METHOD(RedisArray, zscan)
+{
+    ra_generic_scan_cmd(INTERNAL_FUNCTION_PARAM_PASSTHRU, "ZSCAN", sizeof("ZSCAN") - 1);
+}
+
+PHP_METHOD(RedisArray, scan)
+{
+    RedisArray *ra;
+    zend_string *host, *pattern = NULL;
+    zval *object, *redis_inst, *z_iter, z_fun, z_args[3];
+    zend_long count = 0;
+
+    if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "Oz/S|S!l",
+            &object, redis_array_ce, &z_iter, &host, &pattern, &count) == FAILURE) {
+        RETURN_FALSE;
+    }
+
+    if ((ra = redis_array_get(object)) == NULL) {
+        RETURN_FALSE;
+    }
+
+    if ((redis_inst = ra_find_node_by_name(ra, host)) == NULL) {
+        RETURN_FALSE;
+    }
+
+    ZVAL_NEW_REF(&z_args[0], z_iter);
+    if (pattern) ZVAL_STR(&z_args[1], pattern);
+    ZVAL_LONG(&z_args[2], count);
+
+    ZVAL_STRING(&z_fun, "SCAN");
+    call_user_function(&redis_ce->function_table, redis_inst, &z_fun, return_value, ZEND_NUM_ARGS() - 1, z_args);
+    zval_dtor(&z_fun);
+
+    ZVAL_ZVAL(z_iter, &z_args[0], 0, 1);
+}
+
 PHP_METHOD(RedisArray, multi)
 {
     zval *object;
