@@ -47,7 +47,14 @@ class Redis_Cluster_Test extends Redis_Test {
     public function testSwapDB() { return $this->markTestSkipped(); }
     public function testConnectException() { return $this->markTestSkipped(); }
     public function testTlsConnect() { return $this->markTestSkipped(); }
+    public function testReset() { return $this->markTestSkipped(); }
     public function testInvalidAuthArgs() { return $this->markTestSkipped(); }
+    public function testScanErrors() { return $this->markTestSkipped(); }
+
+    /* These 'directed node' commands work differently in RedisCluster */
+    public function testConfig() { return $this->markTestSkipped(); }
+    public function testFlushDB() { return $this->markTestSkipped(); }
+    public function testFunction() { return $this->markTestSkipped(); }
 
     /* Session locking feature is currently not supported in in context of Redis Cluster.
        The biggest issue for this is the distribution nature of Redis cluster */
@@ -269,7 +276,7 @@ class Redis_Cluster_Test extends Redis_Test {
 
         /* We should now have both prefixs' keys */
         foreach ($arr_keys as $str_prefix => $str_id) {
-            $this->assertTrue(in_array("${str_prefix}${str_id}", $arr_scan_keys));
+            $this->assertTrue(in_array("{$str_prefix}{$str_id}", $arr_scan_keys));
         }
     }
 
@@ -372,7 +379,7 @@ class Redis_Cluster_Test extends Redis_Test {
         // watch and unwatch
         $this->redis->watch('x');
         $r->incr('x'); // other instance
-        $this->redis->unwatch('x'); // cancel transaction watch
+        $this->redis->unwatch(); // cancel transaction watch
 
         // This should succeed as the watch has been cancelled
         $ret = $this->redis->multi()->get('x')->exec();
@@ -751,5 +758,25 @@ class Redis_Cluster_Test extends Redis_Test {
             return 'seed[]=' . $host;
         }, self::$_arr_node_map)) . ($auth ? "&$auth" : '');
     }
+
+    /* Test correct handling of null multibulk replies */
+    public function testNullArray() {
+        $key = "key:arr";
+        $this->redis->del($key);
+
+        foreach ([false => [], true => NULL] as $opt => $test) {
+            $this->redis->setOption(Redis::OPT_NULL_MULTIBULK_AS_NULL, $opt);
+
+            $r = $this->redis->rawCommand($key, "BLPOP", $key, .05);
+            $this->assertEquals($test, $r);
+
+            $this->redis->multi();
+            $this->redis->rawCommand($key, "BLPOP", $key, .05);
+            $r = $this->redis->exec();
+            $this->assertEquals([$test], $r);
+        }
+
+        $this->redis->setOption(Redis::OPT_NULL_MULTIBULK_AS_NULL, false);
+    }
 }
 ?>
